/**********************************************************************
 *
 * ffmpeg.c
 *
 * This software is distributed under the GNU Public License version 2
 * See also the file 'COPYING'.
 *
 **********************************************************************/


#include <stdio.h>
#include <malloc.h>
#include <string.h>

#ifdef HAVE_FFMPEG

#include "ffmpeg.h"

void ffmpeg_init(void)
{
	avcodec_init();
	register_avcodec(&mpeg1video_encoder);
}

ffmpeg_t *ffmpeg_open(char *filename, unsigned char *y, unsigned char *u, unsigned char *v, int width, int height, int rate, int bps)
{
	AVCodec *codec;
	AVCodecContext *c;
  
	ffmpeg_t *ffmpeg;
  
	if ((ffmpeg = malloc(sizeof(ffmpeg_t))) == NULL) return NULL;
	memset(ffmpeg, 0, sizeof(*ffmpeg));

	/* find the mpeg1 video encoder */
	codec = avcodec_find_encoder(CODEC_ID_MPEG1VIDEO);
	if (!codec) return NULL;

	c = &ffmpeg->codec_context;

	/* put default values */
	memset(c, 0, sizeof(*c));

	/* put sample parameters */
	c->bit_rate = bps; /* bit per second */

	/* resolution must be a multiple of two */
	c->width  = width;  
	c->height = height;

	/* frames per second */
	c->frame_rate = rate * FRAME_RATE_BASE;  /* default rate is only 5 frames per second */
	c->gop_size = 10; /* emit one intra frame every ten frames */

	/* open it */
	if (avcodec_open(c, codec) < 0) {
		return NULL;
	}    
    
	if (!(ffmpeg->f = fopen(filename, "ab"))) {
		return NULL;
	}    
	/* alloc image and output buffer */

	ffmpeg->outbuf_size = width * height * 10; // large outbuf_buf
	if ((ffmpeg->outbuf = (UINT8*) malloc(ffmpeg->outbuf_size)) == NULL) return NULL;

	ffmpeg->picture.data[0] = y;
	ffmpeg->picture.data[1] = u;
	ffmpeg->picture.data[2] = v;
	ffmpeg->picture.linesize[0] = c->width;
	ffmpeg->picture.linesize[1] = c->width / 2;
	ffmpeg->picture.linesize[2] = c->width / 2;

	return ffmpeg;
}

void ffmpeg_close(ffmpeg_t *ffmpeg)
{
	/* add sequence end code to have a real mpeg file */
	ffmpeg->outbuf[0] = 0x00;
	ffmpeg->outbuf[1] = 0x00;
	ffmpeg->outbuf[2] = 0x01;
	ffmpeg->outbuf[3] = 0xb7;
	fwrite(ffmpeg->outbuf, 1, 4, ffmpeg->f);
	fclose(ffmpeg->f);
	free(ffmpeg->outbuf);
	avcodec_close(&ffmpeg->codec_context);
	free(ffmpeg);
}

void ffmpeg_put_image(ffmpeg_t *c) 
{
	int out_size;

	out_size = avcodec_encode_video(
	    &c->codec_context, 
	    c->outbuf, 
	    c->outbuf_size, 
	    &c->picture);

	fwrite(c->outbuf, 1, out_size, c->f);
}

#endif /* HAVE_FFMPEG */
